package com.ibm.edp.cic.core.ant;

import java.io.File;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.types.DirSet;
import org.eclipse.osgi.service.resolver.VersionRange;
import org.osgi.framework.Version;

import com.ibm.cic.ant.jar.BaseTask;
import com.ibm.cic.ant.jar.OfferingId;
import com.ibm.cic.ant.jar.FixId;
import com.ibm.cic.ant.jar.UrlRepository;
import com.ibm.cic.common.core.repository.IRepository;
import com.ibm.cic.common.core.repository.RepositoryGroup;
import com.ibm.cic.common.core.repository.UnspecifiedUtils;
import com.ibm.edp.cic.core.helper.Util;

public abstract class AbstractCicTaskWrapper extends BaseTask 
{		

	public static final Version versionUnspecified = UnspecifiedUtils.versionUnspecified;
	public static final VersionRange toleranceUnspecified =  UnspecifiedUtils.toleranceUnspecified;
	
    private File destMetadataDir;
    private File destArtifactDir;
    
    private Vector offerings;
    private Vector fixes;
    
    public void setDestMetadataDir(File destMetadataDir) {
        this.destMetadataDir = destMetadataDir;
    }

    public File getDestMetadataDir() {
        return destMetadataDir;
    }
    
    public void setDestArtifactDir(File destArtifactDir) {
        this.destArtifactDir = destArtifactDir;
    }

    public File getDestArtifactDir() {
        return destArtifactDir;
    }
    
    public void addDirRepository(DirSet dirRepositorySet) {
        super.addSrcRepository(dirRepositorySet);
    }

    public void addUrlRepository(UrlRepository urlRepository) {
        super.addSrcRepository(urlRepository);
    }

    public void addOffering(OfferingId offeringId) {
        if (offerings == null) {
            offerings = new Vector();
        }
        offerings.add(offeringId);
    }

    public Vector getOfferings() {
        return offerings;
    }            
    
    public OfferingId getFirstOffering() {
        return getOffering(0);
    }

    public OfferingId getOffering(int idx) {
      OfferingId rc = null;
      if (offerings != null && idx < offerings.size()) 
      {
        rc = (OfferingId)offerings.get( idx );    
      }
      return rc;
    }

    public void addFix(FixId fixId) {
        if (fixes == null) {
            fixes = new Vector();
        }
        fixes.add(fixId);
    }
    
    public Vector getFixes() {
        return fixes;
    }
    
    public FixId getFirstFix() {
        return getFix(0);
    }

    public FixId getFix(int idx) {
      FixId rc = null;
      if (fixes != null && idx < fixes.size()) 
      {
        rc = (FixId)fixes.get( idx );    
      }
      return rc;
    }

    public void openSourceRepositories() {
        if (srcRepositories == null) {
            return;
        }
        for (Iterator iter = srcRepositories.iterator(); iter.hasNext();) {
            Object o = iter.next();
            String srcRepository = null;
            if (o instanceof UrlRepository) {
                UrlRepository urlRepository = (UrlRepository) o;
                srcRepository = urlRepository.getPath();
                Util.openRepository(getRepositoryGroup(), srcRepository);
            } else if (o instanceof DirSet) {
                DirSet dirRepositorySet = (DirSet) o;
                DirectoryScanner ds = dirRepositorySet.getDirectoryScanner( getProject() );
                File baseDir = ds.getBasedir();
                String[] includedDirectories = ds.getIncludedDirectories();
                for (int i = 0; i < includedDirectories.length; i++) {
                    String repositoryPath = includedDirectories[i];
                    File repositoryDir = new File(baseDir, repositoryPath);
                    srcRepository = repositoryDir.getAbsolutePath();
                    Util.openRepository(getRepositoryGroup(), srcRepository);
                }
            }
        }
    }

    protected abstract void doExecute() throws BuildException;
    
	protected void print(List lst)
	{
	   if(lst == null) return; 	
	   for (Iterator iter = lst.iterator(); iter.hasNext();) {
		String logEntry = (String) iter.next();
		log( logEntry, Project.MSG_INFO );
	   }
	}	
	
	Hashtable unspecifiedHash = new Hashtable();  
	
    protected UnspecifiedUtils getUnspecifiedUtils(IRepository repo) {
    	RepositoryGroup repoGroup = new RepositoryGroup( repo.getName() );
    	repoGroup.addExistingRepository(repo, true);
    	UnspecifiedUtils rc = new UnspecifiedUtils(repoGroup);
    	unspecifiedHash.put(rc, repoGroup);
    	return rc;
    }
    
    protected void releaseUnspecifiedUtils(UnspecifiedUtils uu) {
    	
    	RepositoryGroup repoGroup =  (RepositoryGroup)unspecifiedHash.get( uu );
    	if( repoGroup != null )
    	{
    		//System.out.println("Release UU: " + repoGroup.getName());
    		repoGroup.removeAllRepositories();
    		unspecifiedHash.remove( uu );
    	}    		
    } 
    
    protected void resetRepositoryGroup()
    {
       super.resetRepositoryGroup();
       
       //Release all created uncpecified utils
       for (Iterator iter = unspecifiedHash.keySet().iterator(); iter.hasNext();) {
		UnspecifiedUtils uu = (UnspecifiedUtils) iter.next();
		releaseUnspecifiedUtils( uu );		
	   }
       unspecifiedHash.clear();
       unspecifiedHash = null;
    }

}
