package com.ibm.edp.cic.core.ant;

import java.util.Iterator;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.osgi.service.resolver.VersionRange;
import org.osgi.framework.Version;

import com.ibm.cic.ant.jar.FixId;
import com.ibm.cic.common.core.model.IIdentity;
import com.ibm.cic.common.core.model.IFix;
import com.ibm.cic.common.core.model.IIncludedShareableEntity;
import com.ibm.cic.common.core.repository.IRepository;
import com.ibm.edp.cic.core.helper.CicHelper;
import com.ibm.edp.cic.core.helper.Util;

public abstract class AbstractCicTransformFixMetadata extends AbstractCicTaskWrapper 
{	
	private boolean commit = false;

    public void setCommit(boolean b)
    {
    	this.commit = b;
    }

    public boolean getCommit()
    {
    	return this.commit;
    }
    
    protected void doExecute() throws BuildException {

        if ( getDestMetadataDir() == null)
            log("destMetadataDir:null");//$NON-NLS-1$
        else
            log("destMetadataDir:" + getDestMetadataDir().toString());//$NON-NLS-1$
        
        dumpIds("Fixes", getFixes() );//$NON-NLS-1$
        
        createRepositoryGroup("CicTransformFixMetadata"); //$NON-NLS-1$
        
        try {
        	
        	openSourceRepositories();
        	
        	IProgressMonitor monitor =  new NullProgressMonitor();
        	
            IRepository targetRepo = Util.obtainRepository( getDestMetadataDir().getAbsolutePath() );
            
            FixId fixid = getFirstFix();

			if(fixid == null)
			{
                List fixes = targetRepo.getAllFixes( monitor );
                for (Iterator iter = fixes.iterator(); iter.hasNext();) 
                {
					IFix fix = (IFix) iter.next();
					log("Fix: [" + fix.getIdentity().getId() + ", " + fix.getVersion() + "]");//$NON-NLS-1$
					List logs = transform(fix, targetRepo, monitor);
					print( logs );
				}
			}
			else
			{
				IIdentity fixId = CicHelper.getIdentity( fixid.getId() );

				Version fixVersion = fixid.getVersion();
				if(fixVersion == null) fixVersion = versionUnspecified;

				VersionRange fixTolerance = fixid.getTolerance();
				if(fixTolerance == null) fixTolerance = toleranceUnspecified;				
				
				//log("fixVersion=" + fixVersion);
				IFix fix = getUnspecifiedUtils( targetRepo ).findFix(fixId, fixVersion, fixTolerance, monitor);				
				if(fix == null) throw new BuildException("Cannot locate fix: [" + fixid.getId() + ", " + fixVersion + "]");//$NON-NLS-1$
							
				log("Fix: [" + fix.getIdentity().getId() + ", " + fix.getVersion() + "]");//$NON-NLS-1$
				List logs = transform(fix, targetRepo, monitor);
				print( logs );				
			}
            
		} catch (Exception e) {
			e.printStackTrace();
			throw new BuildException( e );
		}          
        finally {
            resetRepositoryGroup();
        }		
    }	
	
	protected void reduceIncludedSE(IIncludedShareableEntity ise, boolean removeSelectors) {
		//Setting unspecified version and tolerance to resolve current SE from build repository
		ise.setVersion( versionUnspecified );
		ise.setTolerance( toleranceUnspecified );
   
		//Removing embedded SE
		ise.setShareableEntity( null );
		
		if( removeSelectors )
		{
			//Removing embedded selectors
			for (Iterator sit = ise.getIncludedShareableEntitySelectors().iterator(); sit.hasNext();){
				sit.next();
				sit.remove();			    	
			}
		}
	}    
    
	protected abstract List transform(IFix offering, IRepository repo, IProgressMonitor monitor) throws Exception;	
}
