package com.ibm.edp.cic.core.helper;




import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.cic.common.core.model.IAssembly;
import com.ibm.cic.common.core.model.IContent;
import com.ibm.cic.common.core.model.IContentSelector;
import com.ibm.cic.common.core.model.IFeature;
import com.ibm.cic.common.core.model.IFeatureBase;
import com.ibm.cic.common.core.model.IFeatureGroup;
import com.ibm.cic.common.core.model.IInstallableUnit;
import com.ibm.cic.common.core.model.IOffering;
import com.ibm.cic.common.core.model.IShareableEntity;
import com.ibm.cic.common.core.model.adapterdata.IArtifact;
import com.ibm.cic.common.core.model.utils.SelectorContext;
import com.ibm.cic.common.core.model.utils.SelectorExpander;
import com.ibm.cic.common.core.repository.IRepository;
import com.ibm.cic.common.core.repository.RepositoryUtils;

public final class CicUtil 
{

    public static void addContent(IContent content, IRepository repoTo) throws Exception
    {
        repoTo.addContent(content);                
    }

	public static List collectIUs( IShareableEntity se, IProgressMonitor monitor )
	{
	  List rc =  new ArrayList();  
	   for (Iterator it = se.getChildren().iterator(); it.hasNext();) 
	   {
		   
		   Object content = it.next();
	       if(content instanceof IInstallableUnit)
	       {
	          IInstallableUnit iu = (IInstallableUnit)content;
	          rc.add( iu );
	       }
	       /*
	       else
	       {
	    	   System.err.println( "Skipping content: " + content);
	    	   System.err.println( "Skipping content2: " + ((IncludedShareableEntity)content).getContent() );
	       }
	       */
	   }
	  return rc;	
	}

	/*
	public static List collectShareableEntities( IOffering offering, IProgressMonitor monitor )
	{
	   return collectShareableEntities( RepositoryUtils.getContentRepository(offering), monitor );
	}
	*/
	
	public static List collectShareableEntities( IRepository repo, IProgressMonitor monitor )
	{
	  List rc =  new ArrayList();  
	   for (Iterator it = repo.getAllContentElements( monitor ).iterator(); it.hasNext();) 
	   {
		   
		   IContent content = (IContent)it.next();
		   if(content instanceof IAssembly)
		   {
			 // do nothing just skip IAssembly			  
		   } 
		   else if(content instanceof IShareableEntity)
	       {
	    	  IShareableEntity se = (IShareableEntity)content;
	          rc.add( se );
	       }
	   }
	  return rc;	
	}
	
	public static List collectIUs( IOffering offering, IProgressMonitor monitor ) throws Exception
	{
	  List features = new ArrayList();
	  CicUtil.collectFeatures(features, offering.getFeatureGroup());
	
	  Set entitySelectors = new HashSet(features.size());
	  for (Iterator iter = features.iterator(); iter.hasNext();) 
	  {
	      IFeature feature = (IFeature) iter.next();
	      IContentSelector selector = feature.getSelector();	      
	      if (selector != null) 
	      {
	          entitySelectors.add(selector);
	      }
	  }
	
	  IAssembly assembly = offering.getAssembly();
	  if(assembly == null) throw new Exception( "Cannot locate assembly for [" + offering.getIdentity().getId() + "]" );
	  
	  /*
	   * 
	      Add also selectors for features which are not specified in the offering but
	      are specified by its assembly
	   * 
	   */ 
	  entitySelectors.addAll( assembly.getSelectors() );
	  
	
	  SelectorExpander expander = new SelectorExpander(assembly, entitySelectors);
	  SelectorContext context = new SelectorContext(assembly, entitySelectors, offering.getProperties(), true);
	  expander.expand(context, monitor);
	
	  IInstallableUnit[] ius = expander.getIUs();
	  
	  List rc = new ArrayList();
	  List offeringIUs = offering.getInstallableUnits();
	  List assemblyIUs = java.util.Arrays.asList(ius);
	  
	  rc.addAll( offeringIUs );
	  rc.addAll( assemblyIUs );
	  
	  return rc;	
	}

	public static List collectArtifactsToDelete(IOffering offering, IRepository repo, IProgressMonitor monitor) throws Exception
	{
		List rc = new ArrayList();
		List sharedArtifacts = new ArrayList();
		
		for(Iterator it = repo.getAllOfferings(monitor).iterator(); it.hasNext(); )
		{
			IOffering iOffering = (IOffering)it.next();
			if( !offering.equals(iOffering) )
			{
               //Offering must be resolved first to obtain reference to content repository
			   RepositoryUtils.resolve(iOffering, monitor);	
			   sharedArtifacts.addAll( collectArtifacts(iOffering, monitor) );
			}
		}
				
		List artifacts = collectArtifacts(offering, monitor);
		for(Iterator it = artifacts.iterator(); it.hasNext(); )
		{
			IArtifact iArtifact = (IArtifact)it.next();
			
			if(!sharedArtifacts.contains(iArtifact))
			{
				rc.add( iArtifact );
			}
			else
			{
				System.out.println("skip: " + iArtifact.getKey().getId() + "," + iArtifact.getKey().getVersion() );
			}
		}		
		
		return rc;
	}
	
	public static List collectArtifacts(IOffering offering, IProgressMonitor monitor) throws Exception 
	{
	    List artifacts = new ArrayList();        
	    List ius = collectIUs(offering, monitor);
		for(Iterator it = ius.iterator(); it.hasNext();) 
		{
			IInstallableUnit iu = (IInstallableUnit) it.next();
	        for (Iterator j = iu.getAdapterData().getArtifacts().iterator(); j.hasNext();) 
	        {
	            IArtifact artifact = (IArtifact) j.next();
	            artifacts.add(artifact);
	        }
	    }
	    return artifacts;
	}

	public static void collectFeatures(List features, IFeatureBase featureBase) 
	{
	    if (featureBase instanceof IFeature) 
	    {
	        IFeature feature = (IFeature) featureBase;
	        
	        features.add(feature);
	        return;
	    } 
	    else if (featureBase instanceof IFeatureGroup) 
	    {
	        IFeatureGroup fgroup = (IFeatureGroup) featureBase;
	        for (Iterator iter = fgroup.getChildren().iterator(); iter.hasNext();) 
	        {
	           IFeatureBase child = (IFeatureBase) iter.next();
	           collectFeatures(features, child);
	        }
	    }
	} 
	
	public static IOffering findOffering(String offeringId, IRepository repo, IProgressMonitor monitor)
	{
	  IOffering rc = null;
    	List offerings = repo.getAllOfferings( monitor );
    	for( Iterator iter = offerings.iterator(); iter.hasNext(); )
    	{
    	   IOffering ioffer = (IOffering)iter.next();   
    	   if(offeringId.equals( ioffer.getIdentity().getId() ) )
    	   {        		  
    		  rc = ioffer;
    		  break;
    	   }
    	}
      return rc;    	
	}
}
