/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2006.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.edp.cic.core.templates.ant;

import java.io.File;
import java.net.URI;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.ProjectHelper;
import org.eclipse.ant.core.AntRunner;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;

import com.ibm.edp.cic.core.EdpCicCorePlugin;

public class AntExecutor 
{
	
	public void executePure( URL url, Map props ) throws Exception
	{
	   File buildFile = new File( new URI(url.toString()) );
	   
	   Project project = new Project();
	   project.setUserProperty("ant.file", buildFile.getAbsolutePath());
	   
	   if( props != null)
	   {
		   for (Iterator iter = props.keySet().iterator(); iter.hasNext();) 
		   {
				String pkey = (String) iter.next();
				String sval = "" + props.get( pkey );
				project.setUserProperty( pkey, sval );
		   }//for		   
	   }//if
	   
	   project.setSystemProperties();
	   project.init();

	   
	   //Create logger
	   project.addBuildListener( new AntConsoleLogger() );
	   
	   
	   //Execute the build file
	   ProjectHelper helper = ProjectHelper.getProjectHelper();
	   project.addReference("ant.projectHelper", helper);
	   helper.parse(project, buildFile);
	   project.executeTarget(project.getDefaultTarget());
	}	
	
	public void executeEclipse(URL url, Map props ) throws Exception
	{
		File buildFile = null;
		
		try
		{
			buildFile = new File( new URI(url.toString()) );
			props.put( "base.template.dir", buildFile.getParentFile().getAbsolutePath() );
		}catch(Throwable t)
		{
			IStatus status = new Status(IStatus.ERROR, EdpCicCorePlugin.getPluginId(), IStatus.CANCEL, "Cannot obtain reference to ant build file: " + url.toString(), t) ;
			throw new CoreException( status );			
		}
		
		IProgressMonitor monitor = new NullProgressMonitor();
		AntRunner runner = new AntRunner();
		
		runner.setBuildFileLocation( buildFile.getAbsolutePath() );
		runner.addUserProperties( props );
		
		//Create logger
		//runner.addBuildLogger( AntConsoleLogger.class.getName() );
		runner.addBuildLogger( "org.apache.tools.ant.DefaultLogger" );
		
		runner.run(monitor);
				
	}
	
}
